/*
   main.c - main function for deckmate program to convert
            80-column punched card decks to PostScript output.

   Author: Paul Hardy, June 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "deckmate.h"


int
main (int argc, char **argv) {

   int  i, j, k;     /* loop variables      */
   int  page_num;    /* Current page number */
   int  line_num;    /* Line of listing on current page; ranges from 1 to 66 */
   int  init = 1;    /* Have not output any lines of text yet                */
   char inbuffer[MAXBUFFER];
   char safe_buffer[2 * MAXBUFFER];   /* To escape characters for PostScript */
   int  output_col;   /* current column in output line */
   int  symbol;       /* line contains one or more glyphs in the Symbol font */
   unsigned fonts_used = FONT_PS_COURIER;   /* can add Helvetica and Symbol  */

   /*
      Variables settable by program arguments.
   */
   char *lang_arg;                      /* language given as argument       */
   char *rgb_arg;                       /* R,G,B color given as argument    */
   double red_arg, green_arg, blue_arg; /* separated colors from rgb_arg    */
   char *infile=NULL, *outfile=NULL;    /* input & output filenames         */
   FILE *infp=NULL, *outfp=NULL;        /* pointers to input & output files */
   struct ps_rgb rgb = {0.75, 1.0, 0.75}; /* default is light green stripes */
   language form_lang = LANG_NONE;   /* for language-specific form markings */
   int  border=1;                /* to print border around page boundary    */
   int  colnums=1;               /* to print column numbers at top of page  */
   int  pagenums=1;              /* to print page numbers at bottom of page */
   int  tab_stops[133]={0};      /* to print dashed lines down entire form  */
   int  ntabs=0;                 /* number of tab stops in tab_stops array  */
   char listing_title [MAXTITLE] = {'\0'}; /* for PostScript header comment */ 
   int  wide=0;                        /* 0 -> 80 columns, 1 -> 132 columns */
   int left_margin, top_margin, box_width, box_height;  /* set from "wide"  */
   int chars_per_line;                  /* 80 if wide == 0, 132 if wide > 0 */
   /*
      fortran_cc = 1 if the first character of a line is to be
      interpreted as a FORTRAN carriage control character, as follows:

            <SP>  (space) - advance to next line
              0   advance two lines
              -   advance three lines
              1   advance to top of next page
              +   no advance; return to start of current line

      Additionally, these are implemented to support some compiler listings:

            <VT>  (Vertical Tab, Control-K) - advance to start of next
                  6-line starting point in listing.

            <FF>  (Form Feed, Control-L) - advance to top of next page

      cc_tape stores vertical positions for a simulated carriage
      control tape.  These can be changed with a comma-separated
      list of lines following the "-C" option on the command line.
   */
   int fortran_cc=0;
   int cc_tape [12] = {1,7,13,19,25,31,37,43,49,55,61,63};

   int error_count;  /* return value from parsing parameters; 0 = no errors */

   /* Parse command line parameters */
   int parse_params (int argc, char **argv,
                     char *infile, char *outfile, FILE **infp, FILE **outfp,
                     language *form_lang,
                     int *border, int *colnums, int *pagenus,
                     struct ps_rgb *rgb, int *tab_stops, int *ntabs,
                     char *listing_title, int *wide,
                     int *fortran_cc, int *cc_tape);

   /* Start new PostScript output file */
   /*
      Parameters: outfp, listing_title, program name, package URL,
                  bounding box width, bounding box height.
   */
   void new_ps_file (FILE *outfp, char *listing_title, char *, char *,
                     int, int);

   /* Prolog section in a PostScript file */
   void new_ps_prolog (FILE *outfp,
                       language form_lang,
                       int border, int colnums, int pagenums,
                       struct ps_rgb *rgb,
                       int *tab_stops, int page_num, int wide);

   /* New PostScript page */
   void new_ps_page (FILE *outfp,
                     language form_lang, int border, int colnums,
                     struct ps_rgb *rgb,
                     int *tab_stops, int page_num, int wide);

   /* End current PostScript page */
   void end_ps_page (FILE *outfp, int page_num, int wide);

   /* End current PostScript output file */
   void end_ps_file (FILE *outfp, int page_num, unsigned fonts_used);

   /* Print one output line */
   void ps_line_out (FILE *outfp, int line_num, char *inbuffer,
                     language form_lang, int wide, int symbol);

   /* Process optional carriage control character to determine new line */
   int get_next_line (char cc_char, int line_num, int init,
                      int fortran_cc, int *cc_tape);

   /* Transform the current input string into an output line */
   void build_line (char *inbuffer, int chars_per_line, int fortran_cc,
                    char *safe_buffer, int *symbol, unsigned *fonts_used);


   error_count =  parse_params (argc, argv, infile, outfile, &infp, &outfp,
                                &form_lang, &border, &colnums, &pagenums,
                                &rgb, tab_stops, &ntabs, listing_title,
                                &wide, &fortran_cc, cc_tape);

   /* If a language for a form template is specified, it will use Helvetica. */
   if (form_lang != LANG_NONE) fonts_used |= FONT_PS_HELVETICA;

   if (wide == 0) {  /* 80 column output */
      left_margin = LEFT_MARGIN_80;
       top_margin =  TOP_MARGIN_80;
       box_width  =   BOX_WIDTH_80;
       box_height =  BOX_HEIGHT_80;
       chars_per_line = 80;
   }
   else {  /* 132 column output */
      left_margin = LEFT_MARGIN_132;
       top_margin =  TOP_MARGIN_132;
       box_width  =   BOX_WIDTH_132;
       box_height =  BOX_HEIGHT_132;
       chars_per_line = 132;
   }

   page_num  = 1;  /* current page number          */
   line_num  = 1;  /* output line of 66 on a page  */

   /* Start new PostScript output file */
   new_ps_file (outfp, listing_title, PACKAGE_STRING, PACKAGE_URL,
                LEFT_MARGIN_80 + BOX_WIDTH_80, TOP_MARGIN_80 + 30);

   new_ps_prolog (outfp, form_lang, border, colnums, pagenums, &rgb,
                  tab_stops, page_num, wide);

   new_ps_page (outfp, form_lang, border, colnums, &rgb, tab_stops,
                page_num, wide);

   while (fgets (inbuffer, MAXBUFFER, infp) != NULL) {

      symbol = 0;  /* line does not yet contain a glyph in the Symbol font */

      /* DEC's PDP-11 Macro Assembler output starts 1st page with one '\r' */
      if (inbuffer [0] == '\r') {
         for (i = 1; inbuffer[i] == '\r'; i++);
         strncpy (&inbuffer[0], &inbuffer[i], MAXBUFFER - 1);
         inbuffer[MAXBUFFER - 1] = '\0';
      }

      /* Handle FORTRAN carriage control, if option is selected */
      if (fortran_cc == 1) {
         line_num = get_next_line (inbuffer[0], line_num, init,
                                   fortran_cc, cc_tape);
      }
      else if (inbuffer[0] == '\v') {  /* <VT>, vertical tab */
         line_num += 6 - (line_num-1) % 6; /* vertical tab stop every 6 lines */
      }
      else if (inbuffer[0] == '\f') {  /* <FF> --> new page */
         line_num = 67;
      }

      /* Skip to next page? */
      if (line_num > 66) {  /* Start a new page after 66 input lines. */
         end_ps_page (outfp, page_num, wide);
         page_num++;
         line_num %= 66;
         new_ps_page (outfp, form_lang, border, colnums,
                      &rgb, tab_stops, page_num, wide);
      }

      /*
         Transform input line (inbuffer) into a PostScript string
         suitable for printing (safe_buffer).
      */
      build_line (inbuffer, chars_per_line, fortran_cc,
                  safe_buffer, &symbol, &fonts_used);

      ps_line_out (outfp, line_num, safe_buffer, form_lang, wide, symbol);
      init = 0;  /* have printed the first line of output */
      if (fortran_cc != 1) line_num++;
   }

   end_ps_page (outfp, page_num, wide);

   end_ps_file (outfp, page_num, fonts_used);

   exit (EXIT_SUCCESS);
}

