/*
   rpg.c - RPG language functions for deckmate program.

   Author: Paul Hardy, June 2020

   Copyright (C) 2020 Paul Hardy

   LICENSE:

      This program is free software: you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published by
      the Free Software Foundation, either version 2 of the License, or
      (at your option) any later version.

      This program is distributed in the hope that it will be useful,
      but WITHOUT ANY WARRANTY; without even the implied warranty of
      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      GNU General Public License for more details.

      You should have received a copy of the GNU General Public License
      along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "deckmate.h"


/*
   Draw invariant form markings for RPG listings.
*/
void
new_ps_page_rpg (FILE *outfp) {

   /*
       Set tabs for RPG Page/Line border (column 2)
       to print a dashed gray line down the page.
   */
   int tab_stops[3]={3,0};

   void new_ps_page_tabs (FILE *outfp, int *tab_stops, int);

   fprintf (outfp, "   %%\n");
   fprintf (outfp, "   %% Template for RPG listings.\n");
   fprintf (outfp, "   %%\n");

   /* Print RPG Page/Line tab before column 3 */
   new_ps_page_tabs (outfp, tab_stops, 0);

   fprintf (outfp, "   /Helvetica findfont 6 scalefont setfont\n");
   fprintf (outfp, "   %d %d moveto 90 rotate\n",
           LEFT_MARGIN_80 + 8, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   (PAGE) show\n");
   fprintf (outfp, "   -90 rotate\n");

   /* Print Line field: 3 columns */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 30, TOP_MARGIN_80 + 20, -22);

   fprintf (outfp, "   %d %d moveto 90 rotate\n",
           LEFT_MARGIN_80 + 23, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   (LINE) show\n");
   fprintf (outfp, "   -90 rotate\n");

   /* Print Form Type field: 1 column */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 36, TOP_MARGIN_80 + 20, -22);

   fprintf (outfp, "   %d %d moveto 90 rotate\n",
           LEFT_MARGIN_80 + 35, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   (TYPE) show\n");
   fprintf (outfp, "   -90 rotate\n");
 
   /* Print Program Identification field: 6 columns */
   fprintf (outfp, "   %d %d moveto 0 %d rlineto stroke\n",
           LEFT_MARGIN_80 + 444, TOP_MARGIN_80 + 20, -BOX_HEIGHT_80 - 20);

   /* Center on line, not rotated */
   fprintf (outfp, "   (PROGRAM) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 462, TOP_MARGIN_80 + 28);
   fprintf (outfp, "   show\n");
   fprintf (outfp, "   (ID) dup stringwidth pop\n");
   fprintf (outfp, "   -2 div %d add %d moveto\n",
            LEFT_MARGIN_80 + 462, TOP_MARGIN_80 + 22);
   fprintf (outfp, "   show\n");

   fprintf (outfp, "   /Helvetica findfont 10 scalefont setfont\n");

   /* Center text on line, adding 3 pt extra for space character (32) */
   fprintf (outfp, "   3 0 32 3 0");

   fprintf (outfp, " (RPG STATEMENT) dup stringwidth pop\n");
   /*
      Adjust starting point to be centered on top of form,
      allowing for 3 pt kerning between 14 letter pairs.
   */
   fprintf (outfp, "   -2 div %f add %d moveto\n",
           LEFT_MARGIN_80 + BOX_WIDTH_80 / 2.0 - 3 * 14 / 2 + 0.5,
           TOP_MARGIN_80 + 22);
   fprintf (outfp, "   awidthshow\n");


   return;
}


/*
   Print context-dependent vertical rules on a line for RPG.
*/
void
ps_line_out_rpg (FILE *outfp,
                 int line_num, char *inbuffer, language form_lang) {

   int i;  /* loop variable */

   int *verticals;  /* Points to one of the arrays of verticals below */
   int h_verticals[81]={5,6,
                        0};
   int f_verticals[81]={5,6,14,
                        15,16,17,18,19,23,27,28,30,31,32,34,38,39,46,52,
                        53,59,65,
                        0};
   int i_verticals[81]={5,6,14,
                        16,17,18,20,24,25,26,27,31,32,33,34,38,39,40,41,
                        42,43,47,51,52,58,60,62,64,66,68,70,
                        0};
   int l_verticals[81]={5,6,14,
                        17,22,27,32,37,42,47,52,57,62,67,72,
                        19,24,29,34,39,44,49,54,59,64,69,
                        0};
   int e_verticals[81]={5,6,8,10,18,26,
                        32,35,39,42,43,44,45,51,54,55,56,57,
                        0};
   int c_verticals[81]={5,6,
                        8,9,11,12,14,15,17,
                        27,32,42,48,51,52,53,55,57,59,
                        0};
   int o_verticals[81]={5,6,14,
                        15,16,17,18,20,22,23,24,25,26,27,28,29,30,31,
                        37,38,39,43,44,70,
                        0};
   int t_verticals[81]={5,6,14,15,16,17,18,19,20,31,32,39,40,47,
                        51,52,54,57,59,60,62,70,
                        0};
   int u_verticals[81]={5,6,7,25,26,27,28,29,30,
                        0};
   int comment_verticals[81]={5,6,
                        0};
   int jcl_verticals[81]={0};


   verticals = NULL;  /* No matching pattern found yet. */

   /*
      Print inter-field vertical lines if not a comment line.
   */
   if ((inbuffer [0] == '/') || (inbuffer [1] == '/')) { /* If IBM JCL */
         verticals = jcl_verticals;
   }
   else {
      if (inbuffer [6] == '*') {
         verticals = comment_verticals;
      }
      else {
         switch (inbuffer [5]) {
            case 'H': case 'h': verticals = h_verticals; break;
            case 'F': case 'f': verticals = f_verticals; break;
            case 'I': case 'i': verticals = i_verticals; break;
            case 'L': case 'l': verticals = l_verticals; break;
            case 'E': case 'e': verticals = e_verticals; break;
            case 'C': case 'c': verticals = c_verticals; break;
            case 'O': case 'o': verticals = o_verticals; break;
            case 'T': case 't': verticals = t_verticals; break;
            case 'U': case 'u': verticals = u_verticals; break;
            default:  break;
         }
      }
      if (verticals != NULL) {
         for (i = 0; verticals[i] != 0; i++) {
            fprintf (outfp, "%d %d tick\n",
                    LEFT_MARGIN_80 + 6 * verticals[i],
                    TOP_MARGIN_80 + 8 - 10 * line_num);
         }
      }
   }

   return;
}

